@@ -10,7 +10,7 @@
 using namespace Rcpp;
 
 
-#define MAX_RETURN 10000000000
+#define MAX_RETURN 10000000000.0
 
 
 
@@ -34,30 +34,28 @@ using namespace Rcpp;
 
 
 
-//' Inline C++ that does most of the work related to the log-likelihood function.
-//' 
-//' See below for the wrapper around this function that replaces the many
-//' arguments required here with one input `XPtr<LL_obj>` object.
-//' 
-//' 
-//' @name cor_phylo_LL_
-//' 
-//' 
-//' @noRd
-//' 
-//' 
-inline double cor_phylo_LL_(const arma::vec& par,
-                            const arma::mat& XX,
-                            const arma::mat& UU,
-                            const arma::mat& MM,
-                            const arma::mat& Vphy,
-                            const arma::mat& tau,
-                            const bool& REML,
-                            const bool& constrain_d,
-                            const double& lower_d,
-                            const bool& verbose,
-                            const double& rcond_threshold) {
+
+// `cor_phylo` log likelihood function.
+// 
+//[[Rcpp::export]]
+double cor_phylo_LL(NumericVector par,
+                    SEXP xptr) {
   
+  XPtr<LogLikInfo> lli(xptr);
+  
+  const arma::mat& XX(lli->XX);
+  const arma::mat& UU(lli->UU);
+  const arma::mat& MM(lli->MM);
+  const arma::mat& Vphy(lli->Vphy);
+  const arma::mat& tau(lli->tau);
+  const bool& REML(lli->REML);
+  const bool& constrain_d(lli->constrain_d);
+  const double& lower_d(lli->lower_d);
+  const bool& verbose(lli->verbose);
+  const double& rcond_threshold(lli->rcond_threshold);
+  
+  
+  bool return_max = false;
   
   uint_t n = Vphy.n_rows;
   uint_t p = XX.n_rows / n;
@@ -66,22 +64,23 @@ inline double cor_phylo_LL_(const arma::vec& par,
   
   arma::mat R = L.t() * L;
   
-  arma::vec d = make_d(par, p, constrain_d, lower_d, true);
-  if (d.n_elem == 0) return MAX_RETURN;
+  arma::vec d = make_d(par, p, constrain_d, lower_d, return_max);
+  if (return_max) return MAX_RETURN;
   
   // OU transform
   arma::mat C = make_C(n, p, tau, d, Vphy, R);
   
   arma::mat V = make_V(C, MM);
-  double rcond_dbl = arma::rcond(V);
+  double rcond_dbl = 0;
+  rcond_dbl = arma::rcond(V);
   if (!arma::is_finite(rcond_dbl) || rcond_dbl < rcond_threshold) return MAX_RETURN;
   
   arma::mat iV = arma::inv(V);
-  arma::mat denom = tp(UU) * iV * UU;
+  arma::mat denom = UU.t() * iV * UU;
   rcond_dbl = arma::rcond(denom);
   if (!arma::is_finite(rcond_dbl) || rcond_dbl < rcond_threshold) return MAX_RETURN;
   
-  arma::mat num = tp(UU) * iV * XX;
+  arma::mat num = UU.t() * iV * XX;
   arma::vec B0 = arma::solve(denom, num);
   arma::mat H = XX - UU * B0;
   
@@ -92,18 +91,18 @@ inline double cor_phylo_LL_(const arma::vec& par,
   
   double LL;
   if (REML) {
-    arma::mat to_det = tp(UU) * iV * UU;
+    arma::mat to_det = UU.t() * iV * UU;
     double det_val;
     arma::log_det(det_val, det_sign, to_det);
-    double lhs = arma::as_scalar(tp(H) * iV * H);
+    double lhs = arma::as_scalar(H.t() * iV * H);
     LL = 0.5 * (logdetV + det_val + lhs);
   } else {
-    LL = 0.5 * arma::as_scalar(logdetV + tp(H) * iV * H);
+    LL = 0.5 * arma::as_scalar(logdetV + H.t() * iV * H);
   }
   
   if (verbose) {
     Rcout << LL << ' ';
-    for (uint_t i = 0; i < par.n_elem; i++) Rcout << par(i) << ' ';
+    for (uint_t i = 0; i < static_cast<uint_t>(par.size()); i++) Rcout << par[i] << ' ';
     Rcout << std::endl;
   }
   
@@ -113,68 +112,26 @@ inline double cor_phylo_LL_(const arma::vec& par,
 
 
 
-//' `cor_phylo` log likelihood function.
-//' 
-//' 
-//' @param par Initial values for the parameters to be optimized over.
-//' @param ll_info_xptr `Rcpp::Xptr` object that points to a C++ `LogLikInfo` object.
-//'     This object stores all the other information needed for the log likelihood
-//'     function.
-//' 
-//' @noRd
-//' 
-//' @name cor_phylo_LL
-//' 
-//[[Rcpp::export]]
-double cor_phylo_LL(const arma::vec& par,
-                     const arma::mat& XX,
-                     const arma::mat& UU,
-                     const arma::mat& MM,
-                     const arma::mat& Vphy,
-                     const arma::mat& tau,
-                     const bool& REML,
-                     const bool& constrain_d,
-                     const double& lower_d,
-                     const bool& verbose,
-                     const double& rcond_threshold) {
+/*
+ Return reciprocal condition numbers for matrices in the log likelihood function.
+ 
+ This function is largely a repeat of the first part of the likelihood function.
+ It is used in the output to guide users wanting to change the `rcond_threshold`
+ argument.
+ */
+std::vector<double> return_rcond_vals(XPtr<LogLikInfo> ll_info) {
   
-  double LL = cor_phylo_LL_(par, XX, UU, MM, Vphy, tau, REML,
-                            constrain_d, lower_d, verbose, rcond_threshold);
-  return LL;
-}
-
-
-
-
-
-//' Return reciprocal condition numbers for matrices in the log likelihood function.
-//' 
-//' This function is largely a repeat of the first part of the likelihood function.
-//' It is used in the output to guide users wanting to change the `rcond_threshold`
-//' argument.
-//' 
-//' 
-//' @param par Initial values for the parameters to be optimized over.
-//' @param ll_info A C++ `LogLikInfo` object storing all the information needed
-//'     for the log likelihood function.
-//' 
-//' @noRd
-//' 
-//' @name return_rcond_vals
-//' 
-std::vector<double> return_rcond_vals(const LogLikInfo& ll_info) {
-  
-  const arma::vec& par(ll_info.min_par);
-  const arma::mat& XX(ll_info.XX);
-  const arma::mat& UU(ll_info.UU);
-  const arma::mat& MM(ll_info.MM);
-  const arma::mat& Vphy(ll_info.Vphy);
-  const arma::mat& tau(ll_info.tau);
-  // const bool& REML(ll_info.REML);
-  const bool& constrain_d(ll_info.constrain_d);
-  const double& lower_d(ll_info.lower_d);
-  // const bool& verbose(ll_info.verbose);
-  // const double& rcond_threshold(ll_info.rcond_threshold);
+  const arma::vec& par(ll_info->min_par);
+  const arma::mat& XX(ll_info->XX);
+  const arma::mat& UU(ll_info->UU);
+  const arma::mat& MM(ll_info->MM);
+  const arma::mat& Vphy(ll_info->Vphy);
+  const arma::mat& tau(ll_info->tau);
+  // const bool& REML(ll_info->REML);
+  const bool& constrain_d(ll_info->constrain_d);
+  const double& lower_d(ll_info->lower_d);
+  // const bool& verbose(ll_info->verbose);
+  // const double& rcond_threshold(ll_info->rcond_threshold);
   
   std::vector<double> rconds_out(2);
   
@@ -191,45 +148,20 @@ std::vector<double> return_rcond_vals(const LogLikInfo& ll_info) {
   arma::mat C = make_C(n, p, tau, d, Vphy, R);
   
   arma::mat V = make_V(C, MM);
-  double rcond_dbl = arma::rcond(V);
+  double rcond_dbl = 0;
+  rcond_dbl = arma::rcond(V);
   rconds_out[0] = rcond_dbl;
   
   arma::mat iV = arma::inv(V);
-  arma::mat denom = tp(UU) * iV * UU;
+  arma::mat denom = UU.t() * iV * UU;
   rcond_dbl = arma::rcond(denom);
   rconds_out[1] = rcond_dbl;
   
   return rconds_out;
 }
 
-// Another version that uses XPtr; keeping it here in case I want to change back
-// //[[Rcpp::export]]
-// double cor_phylo_LL(const arma::vec& par,
-//                     SEXP ll_info_xptr) {
-//   
-//   XPtr<LogLikInfo> ll_info(ll_info_xptr);
-//   double LL = cor_phylo_LL_(par, ll_info->XX, ll_info->UU, ll_info->MM, 
-//                             ll_info->Vphy, ll_info->tau, ll_info->REML, 
-//                             ll_info->constrain_d, ll_info->verbose);
-//   return LL;
-// }
-// The function below could also be useful to keep if you go back to using XPtr:
-// // Combining standard_matrices with LogLikInfo::LogLikInfo to see what gets produced
-// //[[Rcpp::export]]
-// SEXP make_LL_info(const arma::mat& X,
-//                   const std::vector<arma::mat>& U,
-//                   const arma::mat& M,
-//                   const arma::mat& Vphy_,
-//                   const bool& REML_,
-//                   const bool& constrain_d_,
-//                   const bool& verbose_) {
-//   
-//   XPtr<LogLikInfo> ll_info_xptr(new LogLikInfo(X, U, M, Vphy_, REML_, constrain_d_,
-//                                          verbose_));
-//   
-//   return ll_info_xptr;
-// }
-// 
+
+
 
 
 
@@ -248,21 +180,10 @@ std::vector<double> return_rcond_vals(const LogLikInfo& ll_info) {
  ***************************************************************************************
  */
 
-
-//' Fit cor_phylo model using nlopt.
-//'
-//'
-//' @inheritParams ll_info cp_get_output
-//' @inheritParams max_iter cor_phylo
-//' @inheritParams method cor_phylo
-//' 
-//' @return Nothing. `ll_info` is modified in place to have info from the model fit
-//'   after this function is run.
-//'
-//' @name fit_cor_phylo_nlopt
-//' @noRd
-//' 
-void fit_cor_phylo_nlopt(LogLikInfo& ll_info,
+/*
+ Fit cor_phylo model using nlopt.
+ */
+void fit_cor_phylo_nlopt(XPtr<LogLikInfo> ll_info,
                          const double& rel_tol,
                          const int& max_iter,
                          const std::string& method) {
@@ -270,6 +191,9 @@ void fit_cor_phylo_nlopt(LogLikInfo& ll_info,
   Rcpp::Environment nloptr_pkg = Rcpp::Environment::namespace_env("nloptr");
   Rcpp::Function nloptr = nloptr_pkg["nloptr"];
   
+  Rcpp::Environment phyr_pkg = Rcpp::Environment::namespace_env("phyr");
+  Rcpp::Function cor_phylo_LL_fxn = phyr_pkg["cor_phylo_LL"];
+  
   std::string nlopt_algor;
   
   if (method == "nelder-mead-nlopt") nlopt_algor = "NLOPT_LN_NELDERMEAD";
@@ -282,40 +206,33 @@ void fit_cor_phylo_nlopt(LogLikInfo& ll_info,
                               _["xtol_rel"] = 0.0001,
                               _["maxeval"] = max_iter);
   
-  List opt = nloptr(_["x0"] = ll_info.par0,
-                   _["eval_f"] = Rcpp::InternalFunction(&cor_phylo_LL),
+  NumericVector par0(ll_info->par0.begin(), ll_info->par0.end());
+  
+  List opt = nloptr(_["x0"] = par0,
+                   _["eval_f"] = cor_phylo_LL_fxn,
                    _["opts"] = options,
-                   _["XX"] = ll_info.XX,
-                   _["UU"] = ll_info.UU,
-                   _["MM"] = ll_info.MM,
-                   _["Vphy"] = ll_info.Vphy,
-                   _["tau"] = ll_info.tau,
-                   _["REML"] = ll_info.REML,
-                   _["constrain_d"] = ll_info.constrain_d,
-                   _["lower_d"] = ll_info.lower_d,
-                   _["verbose"] = ll_info.verbose,
-                   _["rcond_threshold"] = ll_info.rcond_threshold);
+                   _["xptr"] = Rcpp::wrap(ll_info));
   
-  ll_info.min_par = as<arma::vec>(opt["solution"]);
+  ll_info->min_par = as<arma::vec>(opt["solution"]);
   
-  ll_info.LL = as<double>(opt["objective"]);
+  ll_info->LL = as<double>(opt["objective"]);
   int convcode_ = as<int>(opt["status"]);
   
   if (convcode_ > 0) {
     if (convcode_ < 5) {
-      ll_info.convcode = 0;
+      ll_info->convcode = 0;
     } else {
-      ll_info.convcode = 1;
+      ll_info->convcode = 1;
     }
   } else {
-    ll_info.convcode = -1 * convcode_ + 1;
+    ll_info->convcode = -1 * convcode_ + 1;
   }
 
-  ll_info.iters = as<arma::vec>(opt["iterations"])(0);
+  ll_info->iters = as<arma::vec>(opt["iterations"])(0);
   
-  if (ll_info.verbose) {
-    Rcout << ll_info.LL << ' ';
-    arma::vec& par(ll_info.min_par);
+  if (ll_info->verbose) {
+    Rcout << ll_info->LL << ' ';
+    arma::vec& par(ll_info->min_par);
     for (uint_t i = 0; i < par.n_elem; i++) Rcout << par(i) << ' ';
     Rcout << std::endl;
   }
@@ -324,79 +241,56 @@ void fit_cor_phylo_nlopt(LogLikInfo& ll_info,
 }
 
 
+/*
+ Fit `cor_phylo` model using R's `stats::optim`.
+ 
+ Make sure this doesn't get run in parallel!
 
-//' Fit `cor_phylo` model using R's `stats::optim`.
-//' 
-//' Make sure this doesn't get run in parallel!
-//'
-//'
-//' @inheritParams ll_info_xptr cor_phylo_LL
-//' @inheritParams max_iter cor_phylo
-//' @inheritParams method cor_phylo
-//' 
-//' @return Nothing. `ll_info_xptr` is modified in place to have info from the model fit
-//'   after this function is run.
-//'
-//' @name fit_cor_phylo_R
-//' @noRd
-//' 
-void fit_cor_phylo_R(LogLikInfo& ll_info,
+ */
+void fit_cor_phylo_R(XPtr<LogLikInfo> ll_info,
                      const double& rel_tol,
                      const int& max_iter,
                      const std::string& method,
                      const std::vector<double>& sann) {
-
-  Rcpp::Environment stats = Rcpp::Environment::namespace_env("stats");
+  
+  Rcpp::Environment stats("package:stats"); 
   Rcpp::Function optim = stats["optim"];
   
+  Rcpp::Environment phyr_pkg = Rcpp::Environment::namespace_env("phyr");
+  Rcpp::Function cor_phylo_LL_fxn = phyr_pkg["cor_phylo_LL"];
+  
   Rcpp::List opt;
+
+  NumericVector par0(ll_info->par0.begin(), ll_info->par0.end());
   
   if (method == "sann") {
-    opt = optim(_["par"] = ll_info.par0,
-                _["fn"] = Rcpp::InternalFunction(&cor_phylo_LL),
+    opt = optim(_["par"] = par0,
+                _["fn"] = cor_phylo_LL_fxn,
                 _["method"] = "SANN",
                 _["control"] = List::create(_["maxit"] = sann[0],
                                             _["temp"] = sann[1],
                                             _["tmax"] = sann[2],
                                             _["reltol"] = rel_tol),
-                _["XX"] = ll_info.XX,
-                _["UU"] = ll_info.UU,
-                _["MM"] = ll_info.MM,
-                _["Vphy"] = ll_info.Vphy,
-                _["tau"] = ll_info.tau,
-                _["REML"] = ll_info.REML,
-                _["constrain_d"] = ll_info.constrain_d,
-                _["lower_d"] = ll_info.lower_d,
-                _["verbose"] = ll_info.verbose,
-                _["rcond_threshold"] = ll_info.rcond_threshold);
-    ll_info.par0 = as<arma::vec>(opt["par"]);
+                _["xptr"] = Rcpp::wrap(ll_info));
+    par0 = as<NumericVector>(opt["par"]);
   }
   
-  opt = optim(_["par"] = ll_info.par0,
-              _["fn"] = Rcpp::InternalFunction(&cor_phylo_LL),
+  opt = optim(_["par"] = par0,
+              _["fn"] = cor_phylo_LL_fxn,
               _["method"] = "Nelder-Mead",
               _["control"] = List::create(_["maxit"] = max_iter,
                                           _["reltol"] = rel_tol),
-              _["XX"] = ll_info.XX,
-              _["UU"] = ll_info.UU,
-              _["MM"] = ll_info.MM,
-              _["Vphy"] = ll_info.Vphy,
-              _["tau"] = ll_info.tau,
-              _["REML"] = ll_info.REML,
-              _["constrain_d"] = ll_info.constrain_d,
-              _["lower_d"] = ll_info.lower_d,
-              _["verbose"] = ll_info.verbose,
-              _["rcond_threshold"] = ll_info.rcond_threshold);
+              _["xptr"] = Rcpp::wrap(ll_info));
   
-  ll_info.min_par = as<arma::vec>(opt["par"]);
+  ll_info->min_par = as<arma::vec>(opt["par"]);
   
-  ll_info.LL = as<double>(opt["value"]);
-  ll_info.convcode = as<int>(opt["convergence"]);
-  ll_info.iters = as<arma::vec>(opt["counts"])(0);
+  ll_info->LL = as<double>(opt["value"]);
+  ll_info->convcode = as<int>(opt["convergence"]);
+  ll_info->iters = as<arma::vec>(opt["counts"])(0);
   
-  if (ll_info.verbose) {
-    Rcout << ll_info.LL << ' ';
-    arma::vec& par(ll_info.min_par);
+  if (ll_info->verbose) {
+    Rcout << ll_info->LL << ' ';
+    const arma::vec& par(ll_info->min_par);
     for (uint_t i = 0; i < par.n_elem; i++) Rcout << par(i) << ' ';
     Rcout << std::endl;
   }
@@ -421,26 +315,16 @@ void fit_cor_phylo_R(LogLikInfo& ll_info,
  ***************************************************************************************
  */
 
-
-//' Standardize matrices in place.
-//' 
-//' Makes each column of the `X` matrix have mean of zero and standard deviation of 1.
-//' If `U` isn't empty, this function makes each column in each matrix have
-//' mean of zero and standard deviation of 1, unless all values are the same, in which
-//' case it keeps the standard deviation at zero.
-//' Divides each column of `M` by the original standard deviation of that column in 
-//' `X`.
-//' 
-//' 
-//' @inheritParams X cor_phylo_cpp
-//' @inheritParams U cor_phylo_cpp
-//' @inheritParams M cor_phylo_cpp
-//' 
-//' @return Nothing. Matrices are standardized in place.
-//' 
-//' @name standardize_matrices
-//' @noRd
-//' 
+/*-
+ Standardize matrices in place.
+ 
+ Makes each column of the `X` matrix have mean of zero and standard deviation of 1.
+ If `U` isn't empty, this function makes each column in each matrix have
+ mean of zero and standard deviation of 1, unless all values are the same, in which
+ case it keeps the standard deviation at zero.
+ Divides each column of `M` by the original standard deviation of that column in 
+ `X`.
+ */
 void standardize_matrices(arma::mat& X,
                           std::vector<arma::mat>& U,
                           arma::mat& M) {
@@ -470,24 +354,10 @@ void standardize_matrices(arma::mat& X,
 
 
 
-
-//' Make an `LogLikInfo` object based on input matrices.
-//' 
-//' The output `LogLikInfo` is used for model fitting.
-//' 
-//' @inheritParams X cor_phylo_cpp
-//' @inheritParams U cor_phylo_cpp
-//' @inheritParams M cor_phylo_cpp
-//' @inheritParams Vphy_ cor_phylo_cpp
-//' @inheritParams REML_ cor_phylo_cpp
-//' @inheritParams constrain_d_ cor_phylo_cpp
-//' @inheritParams verbose_ cor_phylo_cpp
-//' 
-//' @return a LogLikInfo that contains info necessary for model fitting
-//' 
-//' @name LogLikInfo
-//' @noRd
-//' 
+/*
+ Make an `LogLikInfo` object based on input matrices.
+ The output `LogLikInfo` is used for model fitting.
+ */
 LogLikInfo::LogLikInfo(const arma::mat& X,
                  const std::vector<arma::mat>& U,
                  const arma::mat& M,
@@ -499,7 +369,7 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
                  const bool& verbose_,
                  const double& rcond_threshold_) 
   : REML(REML_), no_corr(no_corr_), constrain_d(constrain_d_), lower_d(lower_d_),
-    verbose(verbose_),rcond_threshold(rcond_threshold_), iters(0) {
+    verbose(verbose_), rcond_threshold(rcond_threshold_), iters(0) {
   
   uint_t n = Vphy_.n_rows;
   uint_t p = X.n_cols;
@@ -557,7 +427,7 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
   }
   L = arma::cov(eps);
   safe_chol(L, "model fitting");
-  L = tp(L);
+  L = L.t();
   
   par0 = make_par(p, L, no_corr);
   min_par = par0;
@@ -565,37 +435,26 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
 }
 
 
-
-//' Make an `LogLikInfo` object based on input matrices and another LogLikInfo object.
-//' 
-//' The output `LogLikInfo` is used for model fitting.
-//' 
-//' *Note:* This version is used for bootstrapping.
-//' It's different from the one above in that it doesn't re-normalize Vphy, UU, or tau.
-//' If you normalize Vphy and tau twice (which would happen if I used the previous
-//' version of this constructor), it can result in weird behavior.
-//' Notably, the bootstrap replicate will sometimes not converge, but when I output the
-//' same data and re-run cor_phylo on it, it'll converge.
-//' This is confusing, so I'm trying to avoid that.
-//' 
-//' 
-//' @inheritParams X cor_phylo_cpp
-//' @inheritParams U cor_phylo_cpp
-//' @inheritParams M cor_phylo_cpp
-//' @param other Another LogLikInfo object from which to derive much of the information.
-//' 
-//' @return a LogLikInfo that contains info necessary for model fitting
-//' 
-//' @name LogLikInfo
-//' @noRd
-//' 
+/*
+ Make an `LogLikInfo` object based on input matrices and another LogLikInfo object.
+ 
+ The output `LogLikInfo` is used for model fitting.
+ 
+ *Note:* This version is used for bootstrapping.
+ It's different from the one above in that it doesn't re-normalize Vphy, UU, or tau.
+ If you normalize Vphy and tau twice (which would happen if I used the previous
+ version of this constructor), it can result in weird behavior.
+ Notably, the bootstrap replicate will sometimes not converge, but when I output the
+ same data and re-run cor_phylo on it, it'll converge.
+ This is confusing, so I'm trying to avoid that.
+ */
 LogLikInfo::LogLikInfo(const arma::mat& X,
                  const std::vector<arma::mat>& U,
                  const arma::mat& M,
-                 const LogLikInfo& other) 
-  : UU(other.UU), Vphy(other.Vphy), tau(other.tau), REML(other.REML),
-    no_corr(other.no_corr), constrain_d(other.constrain_d), lower_d(other.lower_d),
-    verbose(other.verbose), rcond_threshold(other.rcond_threshold), iters(0) {
+                 XPtr<LogLikInfo> other) 
+  : UU(other->UU), Vphy(other->Vphy), tau(other->tau), REML(other->REML),
+    no_corr(other->no_corr), constrain_d(other->constrain_d), lower_d(other->lower_d),
+    verbose(other->verbose), rcond_threshold(other->rcond_threshold), iters(0) {
 
   uint_t p = X.n_cols;
   
@@ -624,7 +483,7 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
   }
   L = arma::cov(eps);
   safe_chol(L, "a bootstrap replicate");
-  L = tp(L);
+  L = L.t();
   
   par0 = make_par(p, L, no_corr);
   min_par = par0;
@@ -635,56 +494,49 @@ LogLikInfo::LogLikInfo(const arma::mat& X,
 
 
 inline void main_output(arma::mat& corrs, arma::mat& B, arma::mat& B_cov, arma::vec& d,
-                        const LogLikInfo& ll_info,
+                        XPtr<LogLikInfo> ll_info,
                         const arma::mat& X, const std::vector<arma::mat>& U) {
   
   uint_t n = X.n_rows;
   uint_t p = X.n_cols;
   
-  arma::mat L = make_L(ll_info.min_par, p);
+  arma::mat L = make_L(ll_info->min_par, p);
   
   arma::mat R = L.t() * L;
   
   corrs = make_corrs(R);
   
-  d = make_d(ll_info.min_par, p, ll_info.constrain_d, ll_info.lower_d);
+  d = make_d(ll_info->min_par, p, ll_info->constrain_d, ll_info->lower_d);
   
   // OU transform
-  arma::mat C = make_C(n, p, ll_info.tau, d, ll_info.Vphy, R);
+  arma::mat C = make_C(n, p, ll_info->tau, d, ll_info->Vphy, R);
   
-  arma::mat V = make_V(C, ll_info.MM);
+  arma::mat V = make_V(C, ll_info->MM);
   
   arma::mat iV = arma::inv(V);
   
-  arma::mat denom = tp(ll_info.UU) * iV * ll_info.UU;
+  arma::mat denom = ll_info->UU.t() * iV * ll_info->UU;
   
-  arma::mat num = tp(ll_info.UU) * iV * ll_info.XX;
+  arma::mat num = ll_info->UU.t() * iV * ll_info->XX;
   
   arma::vec B0 = arma::solve(denom, num);
   
-  make_B_B_cov(B, B_cov, B0, iV, ll_info.UU, X, U);
+  make_B_B_cov(B, B_cov, B0, iV, ll_info->UU, X, U);
   
   return;
 }
 
 
-//' Retrieve objects for output `cor_phylo` object.
-//' 
-//' @inheritParams X cor_phylo_cpp
-//' @inheritParams U cor_phylo_cpp
-//' @param ll_info an LogLikInfo object that contains info necessary to fit the model.
-//'   After optimization, it contains info from the model fit.
-//' 
-//' @return a list containing output information, to later be coerced to a `cor_phylo`
-//'   object by the `cor_phylo` function.
-//' 
-//' @name cp_get_output
-//' @noRd
-//' 
+/*
+ Retrieve objects for output `cor_phylo` object.
+ 
+ Returns a list containing output information, to later be coerced to a `cor_phylo`
+   object by the `cor_phylo` function.
+ */
 List cp_get_output(const arma::mat& X,
                    const std::vector<arma::mat>& U,
                    const arma::mat& M,
-                   LogLikInfo& ll_info,
+                   XPtr<LogLikInfo> ll_info,
                    const double& rel_tol,
                    const int& max_iter,
                    const std::string& method,
@@ -708,18 +560,18 @@ List cp_get_output(const arma::mat& X,
   main_output(corrs, B, B_cov, d, ll_info, X, U);
   
   double logLik = -0.5 * std::log(2 * arma::datum::pi);
-  if (ll_info.REML) {
-    logLik *= (n * p - ll_info.UU.n_cols);
-    arma::mat to_det = tp(ll_info.XX) * ll_info.XX;
+  if (ll_info->REML) {
+    logLik *= (n * p - ll_info->UU.n_cols);
+    arma::mat to_det = ll_info->XX.t() * ll_info->XX;
     double det_val, det_sign;
     arma::log_det(det_val, det_sign, to_det);
-    logLik += 0.5 * det_val - ll_info.LL;
+    logLik += 0.5 * det_val - ll_info->LL;
   } else {
     logLik *= (n * p);
-    logLik -= ll_info.LL;
+    logLik -= ll_info->LL;
   }
   
-  double k = ll_info.min_par.n_elem + ll_info.UU.n_cols;
+  double k = ll_info->min_par.n_elem + ll_info->UU.n_cols;
   double AIC, BIC;
   AIC = -2 * logLik + 2 * k;
   BIC = -2 * logLik + k * std::log(n / arma::datum::pi);
@@ -755,8 +607,8 @@ List cp_get_output(const arma::mat& X,
     _["logLik"] = logLik,
     _["AIC"] = AIC,
     _["BIC"] = BIC,
-    _["niter"] = ll_info.iters,
-    _["convcode"] = ll_info.convcode,
+    _["niter"] = ll_info->iters,
+    _["convcode"] = ll_info->convcode,
     _["rcond_vals"] = rcond_vals,
     _["bootstrap"] = boot_list
   );
@@ -774,10 +626,7 @@ List cp_get_output(const arma::mat& X,
 //' @param M a n x p matrix with p columns containing standard errors of the trait 
 //'   values in `X`. 
 //' @param Vphy_ phylogenetic variance-covariance matrix from the input phylogeny.
-//' @inheritParams REML cor_phylo
-//' @inheritParams constrain_d cor_phylo
-//' @inheritParams verbose cor_phylo
-//' @inheritParams max_iter cor_phylo
+//' @inheritParams cor_phylo
 //' @param method the `method` input to `cor_phylo`.
 //' 
 //' @return a list containing output information, to later be coerced to a `cor_phylo`
@@ -805,7 +654,8 @@ List cor_phylo_cpp(const arma::mat& X,
   
 
   // LogLikInfo is C++ class to use for organizing info for optimizing
-  LogLikInfo ll_info(X, U, M, Vphy_, REML, no_corr, constrain_d, lower_d, verbose, rcond_threshold);
+  XPtr<LogLikInfo> ll_info(new LogLikInfo(X, U, M, Vphy_, REML, no_corr, constrain_d, 
+                                          lower_d, verbose, rcond_threshold), true);
 
   /*
    Do the fitting.
@@ -819,6 +669,7 @@ List cor_phylo_cpp(const arma::mat& X,
   }
   
   // Retrieve output from `ll_info` object and convert to list
+  // Also do bootstrapping if desired
   List output = cp_get_output(X, U, M, ll_info, rel_tol, max_iter, method,
                               boot, keep_boots, sann);
   
@@ -851,44 +702,43 @@ List cor_phylo_cpp(const arma::mat& X,
 
 
 
-BootMats::BootMats(const arma::mat& X_, const std::vector<arma::mat>& U_,
-                     const arma::mat& M_,
-                     const arma::mat& B_, const arma::vec& d_, const LogLikInfo& ll_info)
+BootMats::BootMats(const arma::mat& X_, 
+                   const std::vector<arma::mat>& U_,
+                   const arma::mat& M_,
+                   const arma::mat& B_, 
+                   const arma::vec& d_, 
+                   XPtr<LogLikInfo> ll_info)
   : X(X_), U(U_), M(M_), X_new(), iD(), X_pred() {
   
-  uint_t n = ll_info.Vphy.n_rows;
+  uint_t n = ll_info->Vphy.n_rows;
   uint_t p = X.n_cols;
   
-  arma::mat L = make_L(ll_info.min_par, p);
+  arma::mat L = make_L(ll_info->min_par, p);
   arma::mat R = L.t() * L;
-  arma::mat C = make_C(n, p, ll_info.tau, d_, ll_info.Vphy, R);
-  arma::mat V = make_V(C, ll_info.MM);
+  arma::mat C = make_C(n, p, ll_info->tau, d_, ll_info->Vphy, R);
+  arma::mat V = make_V(C, ll_info->MM);
   iD = V;
   safe_chol(iD, "bootstrapping-matrices setup");
   iD = iD.t();
   
   // For predicted X values (i.e., without error)
-  X_pred = ll_info.UU;
-  X_pred = tp(arma::conv_to<arma::vec>::from(B_.col(0))) * tp(X_pred);
-  X_pred = tp(X_pred);
+  X_pred = ll_info->UU.t();
+  arma::rowvec tmp = B_.col(0).t();
+  X_pred = tmp * X_pred;
+  X_pred = X_pred.t();
   X_pred.reshape(n, p);
   
   return;
 }
 
-//' Iterate from a BootMats object in prep for a bootstrap replicate.
-//' 
-//' This ultimately updates the LogLikInfo object with new XX and MM matrices,
-//' and updates the BootResults object with the mean and sd.
-//' 
-//' @param ll_info An LogLikInfo object that will inform the next call to the
-//'     log-likelihood function.
-//' @param br A BootResults object that stores output from bootstrapping.
-//' 
-//' @name boot_mats_iterate
-//' @noRd
-//' 
-LogLikInfo BootMats::iterate(const LogLikInfo& ll_info) {
+
+/*
+ Iterate from a BootMats object in prep for a bootstrap replicate.
+ 
+ This ultimately updates the LogLikInfo object with new XX and MM matrices,
+ and updates the BootResults object with the mean and sd.
+ */
+XPtr<LogLikInfo> BootMats::iterate(XPtr<LogLikInfo> ll_info) {
 
   uint_t n = X.n_rows;
   uint_t p = X.n_cols;
@@ -904,7 +754,7 @@ LogLikInfo BootMats::iterate(const LogLikInfo& ll_info) {
   }
   // X_new = X_pred + X_rnd;
 
-  LogLikInfo ll_info_new(X_new, U, M, ll_info);
+  XPtr<LogLikInfo> ll_info_new(new LogLikInfo(X_new, U, M, ll_info), true);
 
   return ll_info_new;
 }
@@ -912,23 +762,23 @@ LogLikInfo BootMats::iterate(const LogLikInfo& ll_info) {
 
 
 // Method to return bootstrapped data
-void BootMats::boot_data(LogLikInfo& ll_info, BootResults& br, const uint_t& i) {
+void BootMats::boot_data(XPtr<LogLikInfo> ll_info, BootResults& br, const uint_t& i) {
   
   br.out_inds.push_back(i+1);
-  br.out_codes.push_back(ll_info.convcode);
+  br.out_codes.push_back(ll_info->convcode);
   
   br.out_mats.push_back(X_new);
   
   return;
 }
 
-void BootMats::one_boot(const LogLikInfo& ll_info, BootResults& br,
+void BootMats::one_boot(XPtr<LogLikInfo> ll_info, BootResults& br,
                         const uint_t& i, const double& rel_tol, const int& max_iter,
                         const std::string& method, const std::string& keep_boots,
                         const std::vector<double>& sann) {
   
   // Generate new data
-  LogLikInfo new_ll_info = iterate(ll_info);
+  XPtr<LogLikInfo> new_ll_info = iterate(ll_info);
   
   /*
    Do the fitting.
@@ -943,7 +793,7 @@ void BootMats::one_boot(const LogLikInfo& ll_info, BootResults& br,
     fit_cor_phylo_nlopt(new_ll_info, rel_tol, max_iter, method);
   }
   // Determine whether convergence failed:
-  bool failed = new_ll_info.convcode != 0;
+  bool failed = new_ll_info->convcode != 0;
   
   if (keep_boots == "all" || (keep_boots == "fail" && failed)) {
     boot_data(new_ll_info, br, i);
